package org.aohe.scanservice.control;

import cn.hutool.core.codec.Base64;
import com.alibaba.fastjson2.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.aohe.exceptions.TwainException;
import org.aohe.scanservice.core.TwainUtils;
import org.aohe.scanservice.result.R;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
public class TwainOperational{
    
    public static String selectOperational(String path) {
        JSONObject json = JSONObject.parse(path);
        
        //操作符
        String function = json.getString("function");
        
        //参数符
        JSONObject param = json.getJSONObject("params");
        
        R r = R.ok();
        try {
            if("001001".equals(function)){
                //获取扫描仪列表
                r = getDeviceList();
            } else if ("001002".equals(function)) {
                //选择扫描仪
                r = setScanner(param.getString("scannerId"));
            }else if ("001003".equals(function)){
                //获取扫描仪操作符列表
                // r = getDeviceOperations();
            }else if ("001004".equals(function)){
                //r = setDeviceOperations();
            }else if ("001007".equals(function)){
                r = setDeviceOperations(param);
            }else if ("001008".equals(function)){
                r = startScan(param);
            }else if ("001012".equals(function)){
                //closeTwSource();
            }else if ("001013".equals(function)){
                //closeTwSource();
            }else if ("001015".equals(function)){
                //r = startScan(param.getString("scannerId"),true);
            }else if ("001016".equals(function)){
                //r = startScan(param.getString("scannerId"),true);
            }
        }catch (Exception e){
            log.info("Error ， ", e);
        }
        
        
        return r.toJsonStr();
    }
    
    /**
     * 001001 获取扫描仪列表
     * @return 返回扫描仪列表
     */
    public static R getDeviceList(){
        try {
            return R.ok(TwainUtils.getDeviceList());
        } catch (TwainException e) {
            log.error("Not found scan list", e);
            return R.fail("Not found scan list");
        }
    }
    
    /**
     * 001002 选择扫描仪
     * @param scannerId 扫描仪id
     * @return 返回成功
     */
    public static R setScanner(String scannerId){
        try {
            TwainUtils.setDevice(scannerId);
            return R.ok();
        } catch (TwainException e) {
            log.error("scan device is not found", e);
            return R.fail("scan device is not found");
        }
    }
    
    /**
     * 001007 设置扫描仪参数
     * @param params 扫描仪参数
     * @return 返回成功
     */
    public static R setDeviceOperations(JSONObject params){
        try {
            TwainUtils.setOption(params);
        } catch (TwainException e) {
            log.error("scan setting is not set", e);
            return R.fail("scan setting is not set, please select scan ");
        }
        return R.ok();
    }
    
    public static R startScan(JSONObject params){
        List<Map<String, String>> base64Files;
        try {
            
            List<File> files;
            if(params != null){
                files = TwainUtils.scan(params.getString("scannerId"), params);
            }else{
                files = TwainUtils.scan();
            }
            
            base64Files = files.stream().map(t -> {
                Map<String, String> map = new HashMap<>();
                map.put(t.getName(), Base64.encode(t));
                return map;
            }).collect(Collectors.toList());
            
        } catch (TwainException e) {
            log.error("scan fail , device has problem", e);
            return R.fail("scan fail , device has problem");
        }
        return R.ok(base64Files);
    }
    
}
