package org.aohe.scanservice.core;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import org.aohe.core.twain.Twain;
import org.aohe.core.twain.TwainScanner;
import org.aohe.exceptions.TwainException;
import org.aohe.scan.Source;
import org.aohe.scan.SourceManager;


import java.io.File;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

public class TwainUtils {
    
    private static String deviceName = null;
    
    private static Collection<Source> sources = null;
    
    private static Source source = null;
    /**
     * 获取设备列表
     * @return 设备列表
     * @throws TwainException 系统错误
     */
    public static List<String> getDeviceList() throws TwainException {
        sources = SourceManager.instance().getSources();
        return sources.stream().map(Source::getName).collect(Collectors.toList());
    }
    
    /**
     * 设置设备
     * @param name 设备名称
     * @throws TwainException 系统错误
     */
    public static void setDevice(String name) throws TwainException {
        if(name != null && !name.equals(deviceName)){
            for(Source s : getSources()){
                if(StrUtil.equals(s.getName(), name)){
                    deviceName = name;
                    source = s;
                    //默认不使用系统UI
                    source.setSystemUI(false);
                    
                    TwainScanner.getScanner().select(name);
                    Twain.getSourceManager().selectSource(name);
                    break;
                }
            }
            if(source == null){
                throw new TwainException("scan device is not found");
            }
        }
    }
    
    /**
     * BW - 0
     * COLOR - 2
     * GRAYSCALE - 1
     * true ->auto
     * @param params
     */
    /**
     * 设置参数
     * @param params 参数 Dpi ->double 0-bw 2-color 1-gray true-auto
     * @throws TwainException 系统错误
     */
    public static void setOption(JSONObject params) throws TwainException {
        if(source == null){
            throw new TwainException("please first set device");
        }
        if(params == null){
            return;
        }
        Double key = params.getDouble("dpi");
        if(key != null){
            source.setDpi(key.intValue());
        }
        Integer mode = params.getInteger("mode");
        if(mode != null){
            switch (mode){
                case 0:
                    source.setColor(Source.ColorMode.BW);
                    break;
                case 1:
                    source.setColor(Source.ColorMode.GRAYSCALE);
                    break;
                case 2:
                default:
                    source.setColor(Source.ColorMode.COLOR);
            }
        }
        
        Boolean paperMode = params.getBoolean("paperMode");
        if(paperMode != null){
            source.setAutoDocumentFeeder(paperMode);
        }
        
        Boolean systemUI = params.getBoolean("systemUI");
        if(systemUI != null){
            source.setSystemUI(systemUI);
        }
    }
    
    /**
     * 扫描
     * @param name 设备名称
     * @param params 参数
     * @return 文件列表
     */
    public static List<File> scan(String name, JSONObject params) throws TwainException {
        source = getSource(name);
        setOption(params);
        return source.scan();
    }
    
    /**
     * 使用已选中的扫描仪扫描
     * @return 文件列表
     */
    public static List<File> scan() throws TwainException {
        return scan(deviceName, null);
    }
    
    /**
     * 获取设备
     * @param name 设备名称
     * @return 设备
     */
    public static Source getSource(String name){
        return sources.stream().filter(source -> source.getName().equals(name)).findFirst().orElse(null);
    }
    
    public static Collection<Source> getSources() throws TwainException {
        if(sources == null || sources.isEmpty()){
            sources = SourceManager.instance().getSources();
        }
        return sources;
    }
    
//    public static void main(String[] args) throws TwainException {
//        Collection<Source> sda = SourceManager.instance().getSources();
//        Source source = sda.stream().findFirst().get();
//        source.setDpi(200);
//        source.setColor(Source.ColorMode.COLOR);
//        List<File> files = source.scan();
//        System.out.println(files.get(0).getName());
//    }
    
    /**
     * 重置
     */
    public static void resource(){
        deviceName = null;
        sources = null;
        source = null;
    }
}
