package org.aohe.scanservice.web;

import cn.hutool.json.JSONUtil;
import cn.hutool.log.Log;
import cn.hutool.log.LogFactory;
import org.aohe.scanservice.control.SaneOperational;
import org.aohe.scanservice.control.TwainOperational;
import org.aohe.scanservice.core.SaneSessionUtils;
import org.aohe.scanservice.core.TwainUtils;
import org.aohe.scanservice.result.R;
import org.aohe.scanservice.utils.SystemUtils;
import org.java_websocket.WebSocket;
import org.java_websocket.handshake.ClientHandshake;
import org.java_websocket.server.WebSocketServer;

import java.net.InetSocketAddress;
import java.net.UnknownHostException;

public class SocketServer extends WebSocketServer {
    
    private static final Log log = LogFactory.get();
    
    public SocketServer(int port) throws UnknownHostException {
        super(new InetSocketAddress(port));
    }
    
    public SocketServer(int port, String path) throws UnknownHostException {
        super(new InetSocketAddress(port));
    }
    
    public SocketServer(InetSocketAddress address) {
        super(address);
    }
    
    @Override
    public void onOpen(WebSocket conn, ClientHandshake handshake) {
        conn.send("Welcome to the scan-server!"); // This method sends a message to the new client
        //broadcast("new connection: " + handshake.getResourceDescriptor()); // This method sends a message to all clients connected
        //System.out.println(conn.getRemoteSocketAddress().getAddress().getHostAddress() + " entered the room!");
        log.info("client is open ,client system is: {}", SystemUtils.getOsName());
    }
    
    @Override
    public void onClose(WebSocket conn, int code, String reason, boolean remote) {
        log.info("client is closed , client system is : {}", SystemUtils.getOsName());
        //这里执行关闭扫描仪连接
        if(SystemUtils.isWindows()){
            TwainUtils.resource();
        } else if (SystemUtils.isLinux()) {
            SaneSessionUtils.resource();
        }
    }
    
    @Override
    public void onMessage(WebSocket conn, String message) {
        log.info("User send Data: \n"+message);
        String result;
        if(!JSONUtil.isTypeJSON(message)){
            result = R.fail("Illegal parameters, not of json type").toJsonStr();
        }else {
            if (SystemUtils.isWindows()) {
                result = TwainOperational.selectOperational(message);
            } else if(SystemUtils.isLinux()) {
                result = SaneOperational.selectOperational(message);
            }else {
                result = R.fail("No corresponding operating system").toJsonStr();
            }
        }
        log.info("Return data: \n"+result);
        conn.send(result);
    }
    
    @Override
    public void onError(WebSocket conn, Exception ex) {
        log.error("error :"+ex.getMessage(),ex);
        if (conn != null) {
            //绑定不到就退出
            //System.exit(0);
            // some errors like port binding failed may not be assignable to a specific
            // websocket
        }
        
    }
    
    @Override
    public void onStart() {
        System.out.println("Server started!");
        setConnectionLostTimeout(0);
        setConnectionLostTimeout(100);
        
    }
}
